<?php

declare(strict_types=1);

/**
 * Copyright OpenSearch Contributors
 * SPDX-License-Identifier: Apache-2.0
 *
 * OpenSearch PHP client
 *
 * @link      https://github.com/opensearch-project/opensearch-php/
 * @copyright Copyright (c) Elasticsearch B.V (https://www.elastic.co)
 * @license   http://www.apache.org/licenses/LICENSE-2.0 Apache License, Version 2.0
 * @license   https://www.gnu.org/licenses/lgpl-2.1.html GNU Lesser General Public License, Version 2.1
 *
 * Licensed to Elasticsearch B.V under one or more agreements.
 * Elasticsearch B.V licenses this file to you under the Apache 2.0 License or
 * the GNU Lesser General Public License, Version 2.1, at your option.
 * See the LICENSE file in the project root for more information.
 */

namespace OpenSearch\Endpoints;

use OpenSearch\Exception\RuntimeException;

/**
 * NOTE: This file is autogenerated using util/GenerateEndpoints.php
 */
class Index extends AbstractEndpoint
{
    public function getURI(): string
    {
        if (!isset($this->index) || $this->index === '') {
            throw new RuntimeException('index is required for index');
        }
        $index = $this->index;
        $id = $this->id ?? null;
        if (isset($id)) {
            return "/$index/_doc/$id";
        }
        return "/$index/_doc";
    }

    public function getParamWhitelist(): array
    {
        return [
            'if_primary_term',
            'if_seq_no',
            'op_type',
            'pipeline',
            'refresh',
            'require_alias',
            'routing',
            'timeout',
            'version',
            'version_type',
            'wait_for_active_shards',
            'pretty',
            'human',
            'error_trace',
            'source',
            'filter_path'
        ];
    }

    public function getMethod(): string
    {
        return 'POST';
    }

    public function setBody($body): static
    {
        if (is_null($body)) {
            return $this;
        }
        $this->body = $body;

        return $this;
    }
}
